/*
  UICLASS.H
  Main header file for the UICLASS source code.


  Copyright (c) 1994 by Borland International
  All Rights Reserved

  This source code is hereby released into the public domain.
  Written by Jeffrey J. Peters

*/




#ifndef __UICLASS_H
#define __UICLASS_H

#include <windows.h>
#include <conio.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
//---------------------------------------------------------------------------
typedef unsigned long  bits32;
typedef unsigned short bits16;
typedef unsigned char  bits08;

#include "keys.h"
//---------------------------------------------------------------------------
char *newStr (char *s);
bits16 GetCharAt (bits16 x, bits16 y);
void PutCharAt (bits16 x, bits16 y, bits16 buf);
int IsValidSel (bits16 s);
int IsValid (bits16 s, bits32 o);
bits32 GetLimit (bits16 s);
bits32 GetBase (bits16 s);
bits08 GetRights (bits16 s);
#ifdef __DPMI32__
  bits08 Get32Rights (bits16 s);
#endif
char *GetMod (bits16);
void MouseShow (void);
void MouseHide (void);
int EvalExpr (char *str, char *result);

extern FILE *log;

//---------------------------------------------------------------------------
// Assembly macros so bcc32 doesn't need to call tasm32 (for speed)
#define LAR_EAX_EBX  {__emit__ (0x0F); __emit__(0x02); __emit__(0xC3);} // lar eax, ebx
#define LSL_EAX_EBX  {__emit__ (0x0F); __emit__(0x03); __emit__(0xC3);} // lsl eax, ebx
#define VERR_BX      {__emit__ (0x0F); __emit__(0x00); __emit__(0xE3);} // verr bx
#define SHL_EAX_16   {__emit__ (0xC1); __emit__(0xE8); __emit__(0x10);} // shr eax, 16
#define MOV_AL_ES_BX {__emit__ (0x26);  \
                      __emit__ (0x67);  \
                      __emit__ (0x8A);  \
                      __emit__ (0x07);} // mov al, es:[bx]
#define PUSH_ES      {__emit__ (0x06);}
#define PUSH_DS      {__emit__ (0x1E);}
#define PUSH_BX      {__emit__ (0x53);}
#define POP_ES       {__emit__ (0x07);}
#define POP_DS       {__emit__ (0x1F);}
#define POP_BX       {__emit__ (0x5B);}

//---------------------------------------------------------------------------
enum COLORSB {
    _BLACK       = (BLACK     << 4),
    _BLUE        = (BLUE      << 4),
    _GREEN       = (GREEN     << 4),
    _CYAN        = (CYAN      << 4),
    _RED         = (RED       << 4),
    _MAGENTA     = (MAGENTA   << 4),
    _BROWN       = (BROWN     << 4),
    _LIGHTGRAY   = (LIGHTGRAY << 4)
};
//---------------------------------------------------------------------------
struct UIPoint
{
  short x, y;
  UIPoint () : x (0), y (0) {}
  UIPoint (short a, short b) : x (a), y (b) {}
  UIPoint (UIPoint &p) : x (p.x), y (p.y) {}
  UIPoint &operator = (UIPoint &p) { x = p.x; y = p.y; return *this;}
  int operator == (UIPoint &p) {return (x == p.x) && (y == p.y);}
  UIPoint &operator++ (int) {x++; y++; return *this;}
  UIPoint &operator-- (int) {x--; y--; return *this;}
  UIPoint &operator+= (UIPoint &p) {x += p.x; y += p.y; return *this;}
  UIPoint &operator-= (UIPoint &p) {x -= p.x; y -= p.y; return *this;}
};
// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =
struct UIRect
{
  UIPoint ul, lr;
  UIRect () : ul (), lr () {}
  UIRect (UIPoint a, UIPoint b) : ul (a), lr (b) {}
  UIRect (short a, short b, short c, short d) : ul (a, b), lr (c, d) {}
  UIRect (UIRect &r) : ul (r.ul), lr (r.lr) {}
  UIRect &operator = (UIRect &r) {ul = r.ul; lr = r.lr; return *this;}
  int operator == (UIRect &r) {return (ul == r.ul) && (lr == r.lr);}
  UIRect &operator++ (int) {ul++; lr++; return *this;}
  UIRect &operator-- (int) {ul--; lr--; return *this;}
  UIRect &operator+= (UIRect &r) {ul += r.ul; lr += r.lr; return *this;}
  UIRect &operator-= (UIRect &r) {ul -= r.ul; lr -= r.lr; return *this;}

  bits16 Area () {return Width() * Height();}
  bits16 Width () {return abs(lr.x-ul.x)+1;}
  bits16 Height () {return abs(lr.y-ul.y)+1;}
  int Contained (UIPoint &p);
};

//---------------------------------------------------------------------------
const unsigned short UIFNoClearScreen = 0x0001;
const unsigned short UIFNoShadow      = 0x0002;
const unsigned short UIFNoCursorFix   = 0x0004;
const unsigned short UIFNoStretch     = 0x0008;
const unsigned short UIFNoScroll      = 0x0010;
const unsigned short UIFShowCursor    = 0x0020;
const unsigned short UIFNoZoom        = 0x0040;

//---------------------------------------------------------------------------
class UIWindow;
class UIMove;
class UIInputLine;
class UIDmp;
class UISel;
class UISpinner;

class UIObject
{
  private:
  void LowLevelIdle (void);
  void Mouse (void);

  public:
  static UIObject *curObj;
  UIObject *prevObj;
#ifdef __DPMI32__
  static HANDLE hPIDUser;
#endif
  static bits16 screenSel;

  UIObject () { prevObj = curObj; curObj = this; }
  virtual ~UIObject () { curObj = prevObj; }

  virtual UIRect GetMaxRect (void) { return UIRect (1,1,80,25); }
  virtual bits16 GetBIOSKey (void);
  virtual int BIOSKeyWaiting (void);
  virtual void ProcessMouse (void);
  virtual int IsContained (UIPoint &p) {return 0;}
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =
class UIWindow : public UIObject
{
  private:

  void MemUnlock ();
  void MemLock ();

  protected:

  friend class UIMove;
  friend class UIInputLine;
  friend class UISel;
  friend class UIDmp;
  friend class UISpinner;

  static char lineType[2][6];

  char *title;
  char *screenSave;
  UIRect rFrame, rWindow;
  UIRect rOrig, rReg;
  bits16 memSize;
  bits16 bgfgScreen, bgfgBorder, uiFlags, shown, zoomed;
  int lineSet;     // 0 = single lines, 1 = double lines
  UIWindow ();     // Used only for derriving

  public:

  UIWindow (UIRect r, char *title);
  ~UIWindow () ;

  virtual void   UISave ();
  virtual void   UIRestore ();
  virtual void   DrawBox (bits16 flags, bits08 bgfgScreen, bits08 bgfgBorder);
  virtual void   ShowTitle (bits08 bgfgTitle);
  virtual void   SlideOver (bits16 dir);
  virtual void   Stretch (int dir);
  virtual void   Zoom (int inOut);
  virtual int    MoveTo (UIRect r);
  virtual void   CallUpdate (int);
  virtual char * GetLineStr (int, int);
  virtual int    SetLineType (int x);
  virtual void   Redraw (int);
  virtual int    Update (int);
  virtual UIRect GetFrame () { return rFrame; }
  virtual bits16 getch ();
  virtual int    kbhit ();
  virtual int    IsZoomed  (void) { return zoomed; }
  virtual bits16 MinWidth  (void) { return 6; }
  virtual bits16 MinHeight (void) { return 4; }
  virtual int    IsContained (UIPoint &p) {return rFrame.Contained (p);}
};
// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UIMove : public UIWindow
{
  UIWindow &parent;
  bits16 minw, minh;
  public:
  UIMove (UIWindow &);
  virtual int Move ();
  virtual bits16 MinWidth  (void) { return minw; }
  virtual bits16 MinHeight (void) { return minh; }

};
// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UISel : public UIWindow
{
  public:

  bits16 sel;
  int lineIndex;
  UISel (UIRect r, char *title);
  virtual int Update (int);
  virtual int begin (void);
  virtual char *Calculate (bits16);
  virtual bits16 MinWidth  (void) { return 25; }
};

class UISelView : public UIWindow
{
  public:

  bits16 sel;
  bits32 limit, base;
  bits08 rights;

  UISelView (bits16);
  virtual int Update (int);
  virtual int begin (void);
  virtual char *CrackRights (bits08);
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =
class UIInputLine : public UIWindow
{
  protected:

  int dispLen;
  char *text;
  bits16 maxLength;
  UIWindow &parent;

  public:

  UIInputLine (UIWindow &w, bits16 len, char *title);
  ~UIInputLine () {}
  virtual bits16 GetLength (void) { return strlen (text); }
  virtual void GetData (char *dest, bits16 maxLen)
  {
    strncpy (dest, text, maxLen);
  }
  virtual char *begin (void);
  virtual int Update (int);
};
// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UIInputShort : public UIInputLine
{
  protected:
  bits16 data;

  public:
  UIInputShort (UIWindow &w, char *title):  data (0),
                UIInputLine (w, 60, title)
  {
  }
  virtual bits16 Value (bits16);
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UIInputLong : public UIInputLine
{
  protected:
  bits32 data;

  public:
  UIInputLong (UIWindow &w, char *title):  data (0),
                UIInputLine (w, 60, title)
  {
  }
  virtual bits32 Value (bits32);
};
// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =
class TDescriptor
{
  public:
  int allocated;
  bits16 index;
  bits32 base;
  bits32 limit;
  bits08 rights;

  TDescriptor (void) : index (0), base (0), limit (0),
                       rights (0), allocated(0) {}
  TDescriptor (bits16 idx) : index (idx), allocated (0)
  {
    base = GetBase (idx);
    limit = GetLimit (idx);
    rights = GetRights (idx);
  }
  ~TDescriptor (void);

  int SetBase (bits32);
  int SetLimit (bits32);
  int SetRights (bits08);
  int Allocate (void);
  int Free (void);
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UISpinner
{
  UIWindow &parent;
  bits16 hold;
  public:
  UISpinner (UIWindow &p) : parent (p), hold (0) {}
  virtual void Spin (int dir);
  virtual void Restore (void);
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UIDmp : UIWindow
{
  protected:

  UISpinner spin;
  TDescriptor &des;
  UIWindow &parent;
  bits16 curSel, srchSel;
  bits32 curOfs, srchOfs;
  int dmpWidth;
  public:
  UIDmp (UIWindow &p, bits16 sel, bits32 ofs);
  ~UIDmp () { delete &des; }
  virtual int Update (int);
  virtual void begin (void);
  virtual bits16 GetSel (void) { return curSel; }
  virtual bits32 GetOfs (void) { return curOfs; }
  virtual void DoNewSelector (bits16 s);
  virtual bits08 GetFarByte (bits16 s, unsigned int o);
  virtual int IsValid (bits16 s, bits32 o);
  virtual bits32 Search (char *srch, bits32 start);
  virtual int ParseSearch (char *s, bits08 *b);
  virtual bits16 MinWidth  (void) { return 32; }
  virtual bits16 MinHeight (void) { return 5; }
};

// =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =  =

class UIMsgBox : public UIWindow
{
  public:

  char *text;
  UIMsgBox (char *string, char *title, int lns = 2, int wid = 40);
  ~UIMsgBox ();
  virtual int Update (int);

};

//---------------------------------------------------------------------------
#endif // __UICLASS_H